package com.example.fractallayout;

import java.util.Random;

import com.mindfusion.diagramming.Behavior;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramNode;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.FractalLayout;
import com.mindfusion.diagramming.Shape;
import com.mindfusion.diagramming.ShapeNode;

import com.mindfusion.drawing.Brush;
import com.mindfusion.drawing.Color;
import com.mindfusion.drawing.GradientBrush;
import com.mindfusion.drawing.SolidBrush;

import android.os.Bundle;
import android.app.Activity;
import android.graphics.RectF;
import android.view.Menu;
import android.view.View;
import android.widget.Button;

public class MainActivity extends Activity
{
	private DiagramView diagramView;
	private Diagram diagram;

	private Random random = new Random();
	private RectF nodeBounds = new RectF(0, 0, 20, 20);
	private Brush[] brushes;

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		diagramView = (DiagramView)findViewById(R.id.diag_view);
		diagram = diagramView.getDiagram();
		initDiagram();
		
		Button btnGenerate = (Button)findViewById(R.id.generate);
        btnGenerate.setOnClickListener(new View.OnClickListener()
        {
			public void onClick(View v)
			{
				randomTree();
			}
		});
	}
	
	private void initDiagram()
	{
		diagram.setLinkHeadShape(null);
		diagram.setDefaultShape(Shape.fromId("Ellipse"));
		diagram.setBackBrush(new SolidBrush(Color.white));

		// allow scrolling by dragging with a single finger
		diagramView.setBehavior(Behavior.Pan);

		// use different brush for each tree level
		brushes = new Brush[]
		{
			new GradientBrush(new Color(176, 196, 222), new Color(138, 43, 226), 40),
			new GradientBrush(Color.white, new Color(173, 216, 230), 40),
			new GradientBrush(Color.white, new Color(0, 191, 255), 40),
			new GradientBrush(new Color(50, 205, 50), Color.green, 40)
		};
	}
	
	private void randomTree()
	{
		diagram.clearAll();

		ShapeNode root = diagram.getFactory().createShapeNode(nodeBounds);
		randomTree(root, 4, 4);
		arrange(root);
	}
	
	private void arrange(DiagramNode root)
	{
		FractalLayout layout = new FractalLayout();
		layout.setRoot(root);
		layout.arrange(diagram);

		diagram.resizeToFitItems(5);
		diagramView.setZoomFactor(40);
		diagramView.bringIntoView(diagram.getNodes().get(1));
	}

	void randomTree(DiagramNode node, int depth, int minChildren)
	{
		if (depth <= 0)
			return;

		Diagram diagram = node.getParent();
		int children = random.nextInt(depth) - 1 + minChildren;

		if (diagram.getNodes().size() < 3 && children < 2)
			children = 2;

		for (int i = 0; i < children; ++i)
		{
			// create child node and link
			ShapeNode child = diagram.getFactory().createShapeNode(nodeBounds);
			child.setBrush(brushes[depth % brushes.length]);
			diagram.getFactory().createDiagramLink(node, child);

			// build child branch
			randomTree(child, depth - 1, minChildren);
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
}