package com.example.treemap;

import java.io.InputStream;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.example.decisionlayout.R;
import com.mindfusion.diagramming.Behavior;
import com.mindfusion.diagramming.ContainerNode;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramNode;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.ShapeNode;
import com.mindfusion.diagramming.TreeMapLayout;

import com.mindfusion.drawing.Color;
import com.mindfusion.drawing.SolidBrush;

import android.os.Bundle;
import android.app.Activity;
import android.graphics.RectF;
import android.view.Menu;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.RadioGroup.OnCheckedChangeListener;


public class MainActivity extends Activity
{
	private Diagram diagram;
	private Color[] scale;
	private RadioButton rbtnWeighByArea;
	private RadioButton rbtnWeighByPopulation;
	private RadioButton rbtnColorByArea;
	private RadioButton rbtnColorByPopulation;
	private final float coef = 3;
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		scale = new Color[]{
				Color.decode("#20C300"),
				Color.decode("#FFFF00"),
				Color.decode("#FF8000")
			};
		
		DiagramView view  = (DiagramView)findViewById(R.id.diag_view);
		view.setBehavior(Behavior.Pan);
		diagram = view.getDiagram();
		diagram.setBackBrush(new SolidBrush(Color.white));
		
		rbtnWeighByArea = (RadioButton)findViewById(R.id.rbtnWeigthsByArea);
		rbtnWeighByPopulation = (RadioButton)findViewById(R.id.rbtnWeightsByPopulation);
		rbtnColorByArea = (RadioButton)findViewById(R.id.rbtnColoredByArea);
		rbtnColorByPopulation = (RadioButton)findViewById(R.id.rbtnColoredByPopulation);
		
		RadioGroup coloredGroup = (RadioGroup) findViewById(R.id.coloredGroup);        
	    coloredGroup.setOnCheckedChangeListener(new OnCheckedChangeListener() 
	    {
	        public void onCheckedChanged(RadioGroup group, int checkedId) {
	            onRadioButtonClicked();
	        }
	    });
	    RadioGroup weigthsGroup = (RadioGroup) findViewById(R.id.weigthsGroup);        
	    weigthsGroup.setOnCheckedChangeListener(new OnCheckedChangeListener() 
	    {
	        public void onCheckedChanged(RadioGroup group, int checkedId) {
	            onRadioButtonClicked();
	        }
	    });

		createTreeMap();
		assignWeights();
		assignColors();
		arrangeDiagram();	
	}
	
	private void createTreeMap()
	{
		diagram.clearAll();

		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = null;
		Document doc = null;
		try
		{
			db = dbf.newDocumentBuilder();
			//File file = new File("Data.xml");		
			InputStream stream = getAssets().open("Data.xml"); 
			doc = db.parse(stream);
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		Element continentsNode = (Element)doc.getDocumentElement();
		NodeList continents = continentsNode.getElementsByTagName("continent");
		for(int i = 0; i<continents.getLength(); i++)
		{
			Element continent = (Element)continents.item(i);
			ContainerNode container = diagram.getFactory().createContainerNode(0,0,10,10);
			container.setTag(continent);
			container.setCaption(continent.getAttribute("name"));
			
			Element countriesNode = (Element)continent.getElementsByTagName("countries").item(0);
			NodeList countries = countriesNode.getElementsByTagName("country");
			for(int j = 0; j<countries.getLength(); j++)
			{
				Element country = (Element)countries.item(j);
				ShapeNode shape = diagram.getFactory().createShapeNode(0,0,10,10);
				shape.setTag(country);
				shape.setText(country.getAttribute("name"));
				
				double area = Double.parseDouble(country.getAttribute("area"));
				double population = Double.parseDouble(country.getAttribute("population"));
				String areaQuantifier = "million sq. km.";
				if(area<1)
				{
					area*=1000;
					areaQuantifier = "thousand sq. km.";
				}
				String populationQuantifier = "billions";
				if(population<1)
				{
					population *= 1000;
					populationQuantifier = "millions";
				}
				
				shape.setToolTip(String.format("%s\nArea: ~%f %s\nPopulation: ~%f %s.",shape.getText(), area, areaQuantifier, population, populationQuantifier));
				container.add(shape);
			}
		}
	}

	private void assignWeights()
	{
		for(DiagramNode node : diagram.getNodes())
		{
			if(node instanceof ContainerNode || node.getLocked())
			{
				continue;
			}
			if(node instanceof ShapeNode)
			{
				if(node.getTag() instanceof Element)
				{
					Element element = (Element)node.getTag();
					if(rbtnWeighByArea.isChecked())
					{
						node.setWeight(Float.parseFloat(element.getAttribute("area")));
					}
					if(rbtnWeighByPopulation.isChecked())
					{
						node.setWeight(Float.parseFloat(element.getAttribute("population")));
					}
				}
				else
				{
					continue;
				}
			}
		}
	}
	
	private void assignColors()
	{
		HashMap<ContainerNode, Double> largestArea = new HashMap<ContainerNode, Double>();
		HashMap<ContainerNode, Double> largestPopulation = new HashMap<ContainerNode, Double>();
		for(DiagramNode node : diagram.getNodes())
		{
			if(node instanceof ContainerNode)
			{
				ContainerNode container = (ContainerNode)node;
				double area = 0.0;
				double population = 0.0;
				for(DiagramNode child : container.getSubordinateGroup().getAttachedNodes())
				{
					if(child.getTag() instanceof Element)
					{
						Element element = (Element)child.getTag();
						area = Math.max(area, Double.parseDouble(element.getAttribute("area")));
						population = Math.max(population, Double.parseDouble(element.getAttribute("population")));
					}
					else continue;
				}
				largestArea.put(container, area);
				largestPopulation.put(container, population);
			}
		}
		for(DiagramNode node:diagram.getNodes())
		{
			if(node instanceof ContainerNode || node.getLocked())
			{
				continue;
			}
			if(node instanceof ShapeNode)//here may be diagram node
			{
				if(node.getTag() instanceof Element)
				{
					Element element = (Element)node.getTag();
					double value = 0;
					double total = 0;
					if(rbtnColorByArea.isChecked())
					{
						value = Double.parseDouble(element.getAttribute("area"));
						total = largestArea.get(ContainerNode.getContainer(node));
					}
					if(rbtnColorByPopulation.isChecked())
					{
						value = Double.parseDouble(element.getAttribute("population"));
						total = largestPopulation.get(ContainerNode.getContainer(node));
					}
					node.setBrush(new SolidBrush(getScaledColor(value, total)));
				}
			}
		}
	}
	
	private Color getScaledColor(double value, double total)
	{
		double factor = value/total;
		if(factor<0)
			factor = 0;
		if(factor>1)
			factor = 1;
		
		double step = 1.0/(scale.length - 1);
		Color start = scale[(int)(factor/step)];
		Color end = scale[Math.min(scale.length-1, (int)(factor/step)+1)];
		
		double factorBase = factor - factor % step;
		return new Color(
				(start.getRed()+(int)((factor-factorBase)*(end.getRed()-start.getRed())/step))%256,
				(start.getGreen()+(int)((factor-factorBase)*(end.getGreen()-start.getGreen())/step))%256,
				(start.getBlue()+(int)((factor-factorBase)*(end.getBlue()-start.getBlue())/step))%256
		);
	}
	
	private void arrangeDiagram()
	{
		TreeMapLayout layout = new TreeMapLayout();
		layout.setPadding(0);
		layout.setContainerPadding(4);
		layout.setLayoutArea(new RectF(0,0,900/coef,540/coef));
		layout.arrange(diagram);
		
		for(DiagramNode node : diagram.getNodes())
		{
			if(node instanceof ShapeNode)
			{
				ShapeNode shape = (ShapeNode)node;
				if(node.getBounds().right-node.getBounds().left < 25/coef || 
						node.getBounds().bottom-node.getBounds().top < 25/coef)
				{
					shape.setText("");
				}
			}
		}
		for(DiagramNode node : diagram.getNodes())
		{
			if(node.getLocked() && ContainerNode.getContainer(node)!=null)
			{
				RectF bounds = node.getBounds();
				bounds.right = ContainerNode.getContainer(node).getBounds().right;
				bounds.bottom = Math.min(30, node.getMasterGroup().getMainItem().getBounds().bottom);
				node.setBounds(bounds);
			}
		}
	}
	
	public void onRadioButtonClicked()
	{
		if(diagram == null)
			return;
		assignWeights();
		assignColors();
		arrangeDiagram();
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

}
