package com.mindfusion.tutorial4;

import java.io.InputStream;
import java.util.Scanner;

import android.content.Context;
import android.graphics.Bitmap;

import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.components.*;
import com.mindfusion.drawing.*;


public class OrgChartNode extends CompositeNode
{
	public OrgChartNode(Diagram diagram, Context context)
	{
		super(diagram);
		
		String xml = loadXmlFile("OrgChartNode.xml", context);
		getComponents().add(XmlLoader.load(xml, null, null));
		findComponents();
		
		title.getTextFormat().setHorizontalAlign(Align.Near);
		fullName.getTextFormat().setHorizontalAlign(Align.Near);
		text.getTextFormat().setHorizontalAlign(Align.Near);
		shape.setIsOutline(true);
		setFill(new SolidBrush(Color.WHITE));
		setStroke(new Pen(2, Color.BLACK));
		setVisible(true);
	}

	// empty constructor required for deserialization
	public OrgChartNode()
	{
	}
	
	public OrgChartNode(OrgChartNode prototype)
	{
		super(prototype);
		findComponents();
	}

	@Override
	protected void onLoad(Diagram diagram)
	{
		super.onLoad(diagram);
		findComponents();
	}
	
	void findComponents()
	{
		// find components by name
		shape = (ShapeComponent) findComponent("Shape");
		image = (ImageComponent) findComponent("Image");
		title = (TextComponent) findComponent("Title");
		fullName = (TextComponent) findComponent("FullName");
		text = (TextComponent) findComponent("Text");
	}
	
	@Override
	protected void saveProperties(DiagramItemProperties props)
	{
		super.saveProperties(props);

		OrgChartNodeProperties state = (OrgChartNodeProperties)props;
		state.Title = getTitle();
		state.FullName = getFullName();
		state.Text = getText();
		state.Stroke = getStroke();
		state.Fill = getFill();
		state.Image = getImage();
	}

	@Override
	protected void restoreProperties(DiagramItemProperties props)
	{
		super.restoreProperties(props);

		OrgChartNodeProperties state = (OrgChartNodeProperties)props;
		setTitle(state.Title);
		setFullName(state.FullName);
		setText(state.Text);
		setStroke(state.Stroke);
		setFill(state.Fill);
		setImage(state.Image);
	}
	
	
	public Pen getStroke()
	{
	    return shape.getPen();
	}
	
	public void setStroke(Pen value)
	{
		shape.setPen(value);
		this.repaint();
	}
	
	public Brush getFill()
	{
	    return shape.getBrush();
	}
	
	public void setFill(Brush value)
	{
		shape.setBrush(value);
	}

	public Bitmap getImage()
	{
		return image.getImage();   
	}
	
	public void setImage(Bitmap value)
	{
		image.setImage(value);
	}

	public String getTitle()
	{
	    return title.getText();
	}
	
	public void setTitle(String value)
	{
		title.setText(value);
	}

	public String getFullName()
	{
	    return fullName.getName();
	}
	
	public void setFullName(String value)
	{
		fullName.setText(value);
	}

	public String getText()
	{
		return text.getText();
	}
	
	public void setText(String value)
	{
		text.setText(value);
	}
	
	private String loadXmlFile(String filepath, Context context)
	{
		InputStream istream = null;		
		String xml = null;
		try
		{
			istream = context.getAssets().open(filepath);
			Scanner myScanner = new Scanner(istream);
			myScanner.useDelimiter("\\A");
			xml = myScanner.next();
			
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		return xml;
	}
	
	private ShapeComponent shape;
	private ImageComponent image;
	private TextComponent title;
	private TextComponent fullName;
	private TextComponent text;
}